<div class="cas-login" style="text-align: center;">
  <p><a href="{% url 'cas_ng_login' %}?next={{ request.GET.next }}">Login with CAS</a></p>
  <p>(Or provide site-specific credentials below)</p>
</div>

