{# customized CAS login snippet with Princeton branding #}
{# use with pu-login.css for styles #}
<div class="cas-login">
  <a href="{% url 'cas_ng_login' %}{% if request.GET.next %}?next={{ request.GET.next }}{% endif %}"><span class="button grp-button">Login with Princeton NetID</span></a>
</div>

<div class="cas-alt"><p>Or login with a local site account:</p></div>

